package com.kob.backend.consumer;

// WebSocket用于前后端通信

import com.alibaba.fastjson2.JSONObject;
import com.kob.backend.consumer.utils.Game;
import com.kob.backend.consumer.utils.JwtAuthenticationUtil;
import com.kob.backend.mapper.UserMapper;
import com.kob.backend.pojo.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.Iterator;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArraySet;

@Component
@ServerEndpoint("/websocket/{token}")   // 注意不要以'/'结尾
public class WebSocketServer {
    /*
        存储所有链接:对所有 websocket 可见的全局变量,存储为 static 静态变量
        因为每个 websocket 实例都在一个独立的线程里,所以该公共变量应该是线程安全的
        使用线程安全的 Hash 表 ConcurrentHashMap<>(),
        将 userId 映射到 webSocketServer
    */
    private static final ConcurrentHashMap<Integer, WebSocketServer> users = new ConcurrentHashMap<>();
    // 使用 CopyOnWriteArraySet 创建一个线程安全的匹配池 matchPool
    private static final CopyOnWriteArraySet<User> matchPool = new CopyOnWriteArraySet<>();
    // 在 WebSocketServer 中注入数据库的方法演示-> 使用 static 定义为独一份的变量
    private static UserMapper userMapper;
    // 后端向前端发送信息,首先需要创建一个 session
    private Session session = null;
    // 用户信息:定义一个成员变量
    private User user;

    // 注入方法
    @Autowired
    public void setUserMapper(UserMapper userMapper) {
        // 静态变量 userMapper 访问需要使用类名 WebSocketServer 访问
        WebSocketServer.userMapper = userMapper;
    }


    // @OnOpen 创建链接时自动触发这个函数
    @OnOpen
    public void onOpen(Session session, @PathParam("token") String token) throws IOException {
        // 建立链接时需要将 session 存下来
        this.session = session;
        // 成功建立连接时,输出 connected!
        System.out.println("backend connected!");
        // 将 userId 取出来,通过 userId 将 user 找出来
        // int userId = Integer.parseInt(token);
        int userId = JwtAuthenticationUtil.getUserId(token);
        this.user = userMapper.selectById(userId);

        // 如果 user 存在, 表示用户登录成功, 用户信息是存在的
        if (this.user != null) {
            // 将 user 存到 users HashMap里
            users.put(userId, this);

            // (测试)后台输出看用户信息
            // System.out.println(user);
        }
        // 否则,断开连接(这里需要抛出异常)
        else {
            this.session.close();
        }


    }

    @OnClose
    public void onClose() {
        // 关闭链接
        System.out.println("backend disconnected!");
        // 断开连接时,需要将 user 从 users 里面删掉
        if (this.user != null) {
            users.remove(this.user.getId());
            // 将匹配池数组删掉
            matchPool.remove(this.user);
        }
    }

    // 开始匹配的逻辑部分
    private void startMatching() {
        System.out.println("start matching");
        matchPool.add(this.user);

        while (matchPool.size()>=2){
            // 迭代器用于枚举前两个人进行匹配
            Iterator<User> it = matchPool.iterator();
            User a = it.next(), b = it.next();
            // 取出两个人之后,从匹配池中将他们删除
            matchPool.remove(a);
            matchPool.remove(b);

            // 匹配成功时,创建联机地图
            Game game = new Game(13,14,20);
            game.createMap();   // 初始化地图

            JSONObject respGameData = new JSONObject();
            respGameData.put("game_map",game.getG());
            respGameData.put("rows",game.getRows());
            respGameData.put("cols",game.getCols());
            respGameData.put("inner_walls_count",game.getInnerWallsCount());

            // 将 a 配对成功的消息传回客户端
            JSONObject respA = new JSONObject();
            respA.put("event","start-matching");
            respA.put("opponent_username",b.getUsername());
            respA.put("opponent_photo",b.getPhoto());
            respA.put("game_data",respGameData);
            // 获取 a 的链接,并通过 sendMessage 将消息传给前端
            users.get(a.getId()).sendMessage(respA.toJSONString());

            // 同理,将 b 的匹配成功信息传回给前端
            JSONObject respB = new JSONObject();
            respB.put("event","start-matching");
            respB.put("opponent_username",a.getUsername());
            respB.put("opponent_photo",a.getPhoto());
            respB.put("game_data",respGameData);
            users.get(b.getId()).sendMessage(respB.toJSONString());
        }
    }

    // 取消匹配的逻辑部分
    private void stopMatching() {
        System.out.println("stop matching");
        matchPool.remove(this.user);
    }

    // @OnMessage 用于从前端接收请求: 一般 onMessage 当成路由使用,做为消息判断处理的中间部分
    @OnMessage
    public void onMessage(String message, Session session) {
        // 从 Client 接收消息
        System.out.println("receive message");
        // 解析从前端接收到的请求
        JSONObject data = JSONObject.parseObject(message);
        String event = data.getString("event");
        // System.out.println(event);

        // 匹配状态函数调用
        if ("start-matching".equals(event)) {
            startMatching();
        } else if ("stop-matching".equals(event)) {
            stopMatching();
        }
    }

    @OnError
    public void onError(Session session, Throwable error) {
        error.printStackTrace();
    }

    // 从后端向前端发送信息
    public void sendMessage(String message) {
        // 异步通信过程,先加一个锁
        synchronized (this.session) {
            try {
                // 将 message 发送到前端
                this.session.getBasicRemote().sendText(message);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}


