package com.kob.backend.consumer.utils;

import java.util.Random;

// 用来管理整个游戏流程
public class Game {
    // 游戏地图:行数,列数,内部障碍物数量
    private final Integer rows;
    private final Integer cols;
    private final Integer inner_walls_count;
    // 地图数组
    private final int[][] g;
    // 定义"上右下左"四个方向的 dx, dy偏移量
    private final int[] dx = {-1, 0, 1, 0}, dy = {0, 1, 0, -1};

    // 初始化构造函数
    public Game(Integer rows, Integer cols, Integer inner_walls_count) {
        this.rows = rows;
        this.cols = cols;
        this.inner_walls_count = inner_walls_count;
        this.g = new int[rows][cols];
    }

    public int getRows(){
        return rows;
    }
    public int getCols(){
        return cols;
    }
    public int getInnerWallsCount(){
        return inner_walls_count;
    }

    // 返回生成的地图
    public int[][] getG() {
        return g;
    }

    // 联通检测方法---true(联通)---false(不通),参数: 起点坐标 sx,sy ,终点坐标 tx,ty
    private boolean check_connectivity(int sx, int sy, int tx, int ty) {
        // 起点就是终点时,结果联通,直接返回 true
        if (sx == tx && sy == ty) return true;
        g[sx][sy] = 1;

        //枚举"上右下左"四个方向,求当前点下一个相邻点的坐标
        for (int i = 0; i < 4; i++) {
            int x = sx + dx[i];
            int y = sy + dy[i];

            // 判断是否撞到障碍物( g[x][y] == 0 表示没有碰到障碍物 ),如果没有赚到障碍物,且可以找到重点的话,返回 true(联通)
            if (x >= 0 && x < this.rows && y >= 0 && y < this.cols && g[x][y] == 0) {
                if (check_connectivity(x, y, tx, ty)) {
                    // 还原状态
                    g[sx][sy] = 0;
                    return true;
                }
            }
        }

        // 还原状态
        g[sx][sy] = 0;
        return false;
    }

    // 画地图
    public boolean draw() {
        // 一开始现将所有障碍物初始化为 false
        for (int r = 0; r < this.rows; r++) {
            for (int c = 0; c < this.cols; c++) {
                // 0 表示空地, 1 表示障碍物(墙)
                g[r][c] = 0;
            }
        }

        // 地图左右边缘障碍物
        for (int r = 0; r < this.rows; r++) {
            g[r][0] = g[r][this.cols - 1] = 1;
        }
        // 地图上下边缘障碍物
        for (int c = 0; c < this.cols; c++) {
            g[0][c] = g[this.rows - 1][c] = 1;
        }

        Random random = new Random();
        // 创建内部随机障碍物,每次计算时会生成两个障碍物,因此这里的循环次数 this.inner_walls_count 需要处以 2
        for (int i = 0; i < this.inner_walls_count / 2; i++) {
            // 避免位置重复:重复遍历 1000 次,只要找到了已经存在障碍物的位置就禁止随机
            for (int j = 0; j < 1000; j++) {
                // 找出本次随机到的行-r 列-c 坐标
                int r = random.nextInt(this.rows);  // random.nextInt(7) 返回 0-7之间的随机整数
                int c = random.nextInt(this.cols);

                // 中心对称:当本坐标或者它的中心对称坐标已经存在障碍物了,则重新计算下一个坐标
                if (g[r][c] == 1 || g[this.rows - 1 - r][this.cols - 1 - c] == 1)
                    continue;

                // 避免内部障碍物覆盖掉左下角和右上角的 A-B 角色出发点,如果是这两个位置,则重新计算下一个坐标
                if (r == this.rows - 2 && c == 1 || r == 1 && c == this.cols - 2)
                    continue;

                // 将计算求得的随机障碍物合法的位置赋值为 1 ,以对该位置进行绘制(包括本坐标及其中心对称坐标)
                g[r][c] = g[this.rows - 1 - r][this.cols - 1 - c] = 1;

                // 1000 次遍历中,规定数量的内部障碍物已经够数之后就 break 掉
                break;
            }
        }

        // 确保 A-B 角色的运动区域是联通的:如果不连通,则直接在创建地图对象之前取消绘制( return false )
        return check_connectivity(this.rows - 2, 1, 1, this.cols - 2);
    }

    public void createMap() {
        // 循环绘制:如果发现哪次循环中画地图成功了,则跳出循环,绘制结束
        for (int i = 0; i < 1000; i++) {
            if (draw())
                break;
        }
    }
}
